#include "aboutdialog.h"
#include "ui_aboutdialog.h"

#include <QGuiApplication>
#include <QPropertyAnimation>
#include <QGraphicsEffect>
#include <QJsonDocument>
#include <QJsonObject>
#include <QDesktopServices>

#include "mainwindow.h"

AboutDialog::AboutDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AboutDialog),
    m_homepage("")
{
    ui->setupUi(this);

    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);

    ui->icon_label->setPixmap(qApp->windowIcon().pixmap((QWindow*)this, QSize(64,64)));
    ui->appName_label->setText(QString("<b>%1</b>").arg(qApp->applicationDisplayName()));
    ui->appVersion_label->setText(QString("Version %1").arg(qApp->applicationVersion()));

    connect(ui->homepage_label, &QLabel::linkActivated, [=](const QString &link) {
        if (link.startsWith("http")) {
            QDesktopServices::openUrl(link);
        } else {
            this->hide();
            qobject_cast<MainWindow*>(parent)->showHelp(link);
        }
    });

    setMinimumSize(sizeHint());
    setMaximumSize(sizeHint());
}

AboutDialog::~AboutDialog()
{
    delete ui;
}

QString AboutDialog::homepage() const
{
    return m_homepage;
}

QUrl AboutDialog::updateUrl() const
{
    return m_updateUrl;
}

void AboutDialog::setHomepage(QString homepage)
{
    if (m_homepage == homepage)
        return;

    m_homepage = homepage;
    ui->homepage_label->setText(QString("<a href=\"%1\">Homepage</a> | <a href=\"license.html\">License</a>").arg(homepage));
    emit homepageChanged(homepage);
}

void AboutDialog::setUpdateUrl(QUrl updateUrl)
{
    if (m_updateUrl == updateUrl)
        return;

    m_updateUrl = updateUrl;
    emit updateUrlChanged(updateUrl);
}

void AboutDialog::on_update_pushButton_clicked()
{
    ui->stackedWidget->setCurrentIndex(1);

    m_reply = m_qnam.get(QNetworkRequest(updateUrl()));
    connect(m_reply, &QNetworkReply::finished, [=]() {
        if (m_reply->error()) {
            ui->stackedWidget->setCurrentIndex(4);
        } else {
            QJsonDocument doc = QJsonDocument::fromJson(m_reply->readAll());
            double latest = doc.object().value("latest").toDouble();
            double current = qApp->applicationVersion().toDouble();
            if (latest <= current) {
                ui->stackedWidget->setCurrentIndex(2);
            } else if (latest > current) {
                ui->stackedWidget->setCurrentIndex(3);
            }
        }
        m_reply->deleteLater();
    });
}
