#ifndef LAYOUTBROWSER_H
#define LAYOUTBROWSER_H

#include <QWidget>
#include <QAbstractItemModel>
#include <QAbstractItemDelegate>
#include <QImage>
#include <QMap>

#include <comiclayout.h>

class LayoutBrowser;
class QPushButton;

class LayoutDelegate : public QAbstractItemDelegate
{
    Q_OBJECT

public:
    explicit LayoutDelegate(QObject *parent = Q_NULLPTR);


    // QAbstractItemDelegate interface
public:
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;

    void setLayoutBrowser(LayoutBrowser *layoutBrowser);

private:
    LayoutBrowser *m_layoutBrowser;
};

class LayoutsTableModel : public QAbstractItemModel
{
    Q_OBJECT

public:
    Q_PROPERTY(int itemsPerRow READ itemsPerRow WRITE setItemsPerRow NOTIFY itemsPerRowChanged)
    Q_PROPERTY(QStringList files READ files WRITE setFiles NOTIFY filesChanged)

    explicit LayoutsTableModel(QStringList files, QObject *parent = Q_NULLPTR);

    // QAbstractItemModel interface
    public:
    QModelIndex index(int row, int column, const QModelIndex &parent) const;
    QModelIndex parent(const QModelIndex &child) const;
    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;

    int itemsPerRow() const;
    void setLayoutBrowser(LayoutBrowser *layoutBrowser);

    QStringList files() const;

public slots:
    void setItemsPerRow(int itemsPerRow);
    void setFiles(QStringList files);

signals:
    void itemsPerRowChanged(int itemsPerRow);
    void filesChanged(QStringList files);

private:
    int m_itemsPerRow;
    LayoutBrowser *m_layoutBrowser;
    QStringList m_files;
};



namespace Ui {
class LayoutBrowser;
}

class LayoutBrowser : public QWidget
{
    Q_OBJECT

public:
    Q_PROPERTY(int orientation READ orientation WRITE setOrientation NOTIFY orientationChanged)

    explicit LayoutBrowser(QWidget *parent = 0);
    ~LayoutBrowser();

    QString selectedLayout() const;
    int orientation() const;

public slots:
    void setOrientation(int orientation);

signals:
    void orientationChanged(int orientation);
    void resized();
    void selectionChanged();

private:
    Ui::LayoutBrowser *ui;

    int m_orientation;

    QList<QPushButton*> m_tagButtons;

private slots:
    void onTagsChanged(bool checked);

    // QWidget interface
protected:
    void resizeEvent(QResizeEvent *evt);
};

#endif // LAYOUTBROWSER_H
