#include "layoutconfiguration.h"
#include "ui_layoutconfiguration.h"

#include <QDebug>

#include <comiclayout.h>

struct LayoutConfiguration::Private {
    QSharedPointer<ComicLayout> comicLayout;
};

LayoutConfiguration::LayoutConfiguration(QWidget *parent) :
    QGroupBox(parent),
    ui(new Ui::LayoutConfiguration),
    m_private(new Private)
{
    ui->setupUi(this);
}

LayoutConfiguration::~LayoutConfiguration()
{
    delete ui;
}

QSharedPointer<ComicLayout> LayoutConfiguration::comicLayout() const
{
    return m_private->comicLayout;
}

void LayoutConfiguration::setComicLayout(QSharedPointer<ComicLayout> comicLayout)
{
    if (m_private->comicLayout == comicLayout)
        return;

    if (!m_private->comicLayout.isNull()) {
        disconnect(comicLayout.data(), SIGNAL(marginsChanged(double)), ui->margins_doubleSpinBox, SLOT(setValue(double)));
        disconnect(ui->margins_doubleSpinBox, SIGNAL(valueChanged(double)), comicLayout.data(), SLOT(setMargins(double)));
        disconnect(comicLayout.data(), SIGNAL(lineStrengthChanged(double)), ui->lineStrength_doubleSpinBox, SLOT(setValue(double)));
        disconnect(ui->lineStrength_doubleSpinBox, SIGNAL(valueChanged(double)), comicLayout.data(), SLOT(setLineStrength(double)));
    }

    m_private->comicLayout = comicLayout;
    connect(comicLayout.data(), SIGNAL(marginsChanged(double)), ui->margins_doubleSpinBox, SLOT(setValue(double)));
    connect(ui->margins_doubleSpinBox, SIGNAL(valueChanged(double)), comicLayout.data(), SLOT(setMargins(double)));
    connect(comicLayout.data(), SIGNAL(lineStrengthChanged(double)), ui->lineStrength_doubleSpinBox, SLOT(setValue(double)));
    connect(ui->lineStrength_doubleSpinBox, SIGNAL(valueChanged(double)), comicLayout.data(), SLOT(setLineStrength(double)));
    ui->margins_doubleSpinBox->setValue(comicLayout->margins());
    ui->lineStrength_doubleSpinBox->setValue(comicLayout->lineStrength());

    emit comicLayoutChanged(comicLayout);
}
