#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFileDialog>
#include <QDebug>

#include <aboutdialog.h>
#include "helpbrowser.h"
#include <comicpage.h>
#include <comiclayout.h>
#include <Generators/Krita/krapagegenerator.h>

struct MainWindow::Private {
    Private() :
        comicPage(new ComicPage(""))
    {}

    QSharedPointer<AboutDialog> aboutDialog;
    QSharedPointer<HelpBrowser> helpBrowser;
    QSharedPointer<ComicPage> comicPage;
};

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    m_private(new Private)
{
    ui->setupUi(this);

    ui->layoutConfiguration->setComicLayout(m_private->comicPage->layout());

    connect(ui->pageSetup, SIGNAL(orientationChanged(int)), ui->layoutBrowser, SLOT(setOrientation(int)));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::showHelp(const QString &link)
{
    if (m_private->helpBrowser.isNull()) {
        m_private->helpBrowser.reset(new HelpBrowser(this));
    }
    m_private->helpBrowser->open(link);
}

QSharedPointer<ComicPage> MainWindow::comicPage() const
{
    return m_private->comicPage;
}

void MainWindow::setComicPage(QSharedPointer<ComicPage> comicPage)
{
    if (m_private->comicPage == comicPage)
        return;

    m_private->comicPage = comicPage;
    emit comicPageChanged(comicPage);
}

void MainWindow::on_pageSetup_changed()
{
    ui->generate_pushButton->setEnabled(ui->pageSetup->isValid() && !ui->layoutBrowser->selectedLayout().isEmpty());
}

void MainWindow::on_generate_pushButton_clicked()
{
    QFileDialog dialog(this);
    dialog.setNameFilters(QStringList() << tr("Krita files (*.kra)") << tr("All files (*.*)"));
    dialog.setDefaultSuffix("kra");
    dialog.setAcceptMode(QFileDialog::AcceptSave);
    dialog.setWindowTitle(tr("Save file"));
    QString filename;

    if (dialog.exec()) {
        filename = dialog.selectedFiles()[0];
    } else {
        return;
    }

    m_private->comicPage->layout()->setFilename(ui->layoutBrowser->selectedLayout());
    m_private->comicPage->setBindingSize(QSize(ui->pageSetup->finishedWidth(), ui->pageSetup->finishedHeight()));
    m_private->comicPage->setManuscriptSize(QSize(ui->pageSetup->manuscriptWidth(), ui->pageSetup->manuscriptHeight()));
    m_private->comicPage->setResolution(ui->pageSetup->resolution());
    m_private->comicPage->setBleedWidth(ui->pageSetup->bleedWidth());

    KraPageGenerator gen;
    gen.setComicPage(m_private->comicPage);
    gen.setAuthor(ui->pageSetup->author());
    gen.setTitle(ui->pageSetup->title());
    gen.generate(filename);
}

void MainWindow::on_layoutBrowser_selectionChanged()
{
    on_pageSetup_changed();
}

void MainWindow::on_about_pushButton_clicked()
{
    if (m_private->aboutDialog.isNull()) {
        m_private->aboutDialog.reset(new AboutDialog(this));
        m_private->aboutDialog->setHomepage("http://moritzmolch.com/2165");
        m_private->aboutDialog->setUpdateUrl(QUrl("http://apps.moritzmolch.com/files/ComicCreator/version.json"));
    }
    m_private->aboutDialog->show();
}

void MainWindow::on_help_pushButton_clicked()
{
    showHelp();
}
