#include "pagesetupwidget.h"
#include "ui_pagesetupwidget.h"

#include <QJsonDocument>
#include <QJsonArray>
#include <QJsonObject>
#include <QJsonValue>
#include <QFile>
#include <QDebug>

PageSetupWidget::PageSetupWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::PageSetupWidget)
{
    ui->setupUi(this);

    connect(ui->comboBox_manuscriptSize, SIGNAL(currentIndexChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->comboBox_finishedSize, SIGNAL(currentIndexChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->comboBox_bleedWidth, SIGNAL(currentIndexChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->comboBox_resolution, SIGNAL(currentIndexChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->comboBox_bleedWidth, SIGNAL(currentIndexChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->spinBox_customFinishedWidth, SIGNAL(valueChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->spinBox_customFinishedHeight, SIGNAL(valueChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->spinBox_customManuscriptWidth, SIGNAL(valueChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->spinBox_customManuscriptHeight, SIGNAL(valueChanged(int)), this, SLOT(paperSize_changed()));
    connect(ui->spinBox_customBleedWidth, SIGNAL(valueChanged(int)), this, SLOT(paperSize_changed()));

    connect(ui->comboBox_manuscriptSize, SIGNAL(currentIndexChanged(int)), this, SIGNAL(changed()));
    connect(ui->comboBox_finishedSize, SIGNAL(currentIndexChanged(int)), this, SIGNAL(changed()));
    connect(ui->comboBox_resolution, SIGNAL(currentIndexChanged(int)), this, SIGNAL(changed()));
    connect(ui->comboBox_colors, SIGNAL(currentIndexChanged(int)), this, SIGNAL(changed()));
    connect(ui->comboBox_orientation, SIGNAL(currentIndexChanged(int)), this, SIGNAL(changed()));
    connect(ui->spinBox_customFinishedWidth, SIGNAL(valueChanged(int)), this, SIGNAL(changed()));
    connect(ui->spinBox_customFinishedHeight, SIGNAL(valueChanged(int)), this, SIGNAL(changed()));
    connect(ui->spinBox_customManuscriptWidth, SIGNAL(valueChanged(int)), this, SIGNAL(changed()));
    connect(ui->spinBox_customManuscriptHeight, SIGNAL(valueChanged(int)), this, SIGNAL(changed()));

    connect(ui->comboBox_orientation, SIGNAL(currentIndexChanged(int)), this, SIGNAL(orientationChanged(int)));

    QFile papersizeFile(":/data/papersizes.json");
    papersizeFile.open(QIODevice::ReadOnly);
    m_paperSizes = QJsonDocument::fromJson(papersizeFile.readAll());
    papersizeFile.close();

    ui->comboBox_manuscriptSize->addItem(tr("Automatic"));

    Q_FOREACH (const QJsonValue &v, m_paperSizes.array()) {
        QJsonObject obj = v.toObject();
        QString name = obj.value("name").toString();
        int width = obj.value("width").toInt();
        int height = obj.value("height").toInt();
        QString entry = QString("%1 (%2 mm x %3 mm)").arg(tr(name.toUtf8())).arg(width).arg(height);
        ui->comboBox_manuscriptSize->addItem(entry);
        ui->comboBox_finishedSize->addItem(entry);
    }

    ui->comboBox_manuscriptSize->addItem(tr("Custom…"));
    ui->comboBox_finishedSize->addItem(tr("Custom…"));

    ui->label_colors->setParent(Q_NULLPTR);
    ui->comboBox_colors->setParent(Q_NULLPTR);
    ui->label_author->setParent(Q_NULLPTR);
    ui->lineEdit_author->setParent(Q_NULLPTR);
    ui->label_title->setParent(Q_NULLPTR);
    ui->lineEdit_title->setParent(Q_NULLPTR);
    ui->label_pageNumber->setParent(Q_NULLPTR);
    ui->spinBox_pageNumber->setParent(Q_NULLPTR);

    paperSize_changed();
}

PageSetupWidget::~PageSetupWidget()
{
    delete ui;
}

int PageSetupWidget::manuscriptWidth() const
{
    if (ui->comboBox_manuscriptSize->currentIndex() == 0) {
        // Automatic selected
        return finishedWidth()+bleedWidth()*2+20;
    } else if (ui->comboBox_manuscriptSize->currentIndex() == (ui->comboBox_manuscriptSize->count()-1)) {
        // Custom
        return ui->spinBox_customManuscriptWidth->value();
    } else {
        if (ui->comboBox_orientation->currentIndex() == 0) {
            return m_paperSizes.array().at(ui->comboBox_manuscriptSize->currentIndex()-1).toObject().value("width").toInt();
        } else {
            return m_paperSizes.array().at(ui->comboBox_manuscriptSize->currentIndex()-1).toObject().value("height").toInt();
        }
    }
}

int PageSetupWidget::manuscriptHeight() const
{
    if (ui->comboBox_manuscriptSize->currentIndex() == 0) {
        // Automatic selected
        return finishedHeight()+bleedWidth()*2+20;
    } else if (ui->comboBox_manuscriptSize->currentIndex() == (ui->comboBox_manuscriptSize->count()-1)) {
        // Custom
        return ui->spinBox_customManuscriptHeight->value();
    } else {
        if (ui->comboBox_orientation->currentIndex() == 0) {
            return m_paperSizes.array().at(ui->comboBox_manuscriptSize->currentIndex()-1).toObject().value("height").toInt();
        } else {
            return m_paperSizes.array().at(ui->comboBox_manuscriptSize->currentIndex()-1).toObject().value("width").toInt();
        }
    }

}

int PageSetupWidget::finishedWidth() const
{
    if (ui->comboBox_orientation->currentIndex() == 0) {
        if (ui->comboBox_finishedSize->currentIndex() == (ui->comboBox_finishedSize->count()-1)) {
            return ui->spinBox_customFinishedWidth->value();
        } else {
            return m_paperSizes.array().at(ui->comboBox_finishedSize->currentIndex()).toObject().value("width").toInt();
        }
    } else {
        if (ui->comboBox_finishedSize->currentIndex() == (ui->comboBox_finishedSize->count()-1)) {
            return ui->spinBox_customFinishedHeight->value();
        } else {
            return m_paperSizes.array().at(ui->comboBox_finishedSize->currentIndex()).toObject().value("height").toInt();
        }
    }

}

int PageSetupWidget::finishedHeight() const
{
    if (ui->comboBox_orientation->currentIndex() == 0) {
        if (ui->comboBox_finishedSize->currentIndex() == (ui->comboBox_finishedSize->count()-1)) {
            return ui->spinBox_customFinishedHeight->value();
        } else {
            return m_paperSizes.array().at(ui->comboBox_finishedSize->currentIndex()).toObject().value("height").toInt();
        }
    } else {
        if (ui->comboBox_finishedSize->currentIndex() == (ui->comboBox_finishedSize->count()-1)) {
            return ui->spinBox_customFinishedWidth->value();
        } else {
            return m_paperSizes.array().at(ui->comboBox_finishedSize->currentIndex()).toObject().value("width").toInt();
        }
    }

}

int PageSetupWidget::resolution() const
{
    if (ui->comboBox_resolution->currentIndex() == (ui->comboBox_resolution->count()-1)) {
        return ui->spinBox_customResolution->value();
    } else {
        return ui->comboBox_resolution->currentText().split(' ')[0].toInt();
    }
}

int PageSetupWidget::bleedWidth() const
{
    if (ui->comboBox_bleedWidth->currentIndex() == (ui->comboBox_bleedWidth->count()-1)) {
        return ui->spinBox_customBleedWidth->value();
    } else {
        return ui->comboBox_bleedWidth->currentText().split(' ')[0].toInt();
    }
}

QString PageSetupWidget::author() const
{
    return ui->lineEdit_author->text();
}

QString PageSetupWidget::title() const
{
    return ui->lineEdit_title->text();
}

int PageSetupWidget::pageNumber() const
{
    return ui->spinBox_pageNumber->value();
}

bool PageSetupWidget::isValid() const
{
    return (manuscriptWidth() >= finishedWidth()) && (manuscriptHeight() >= finishedHeight());
}

void PageSetupWidget::paperSize_changed()
{
    ui->widget_customManuscriptSize->setVisible(ui->comboBox_manuscriptSize->currentIndex() == (ui->comboBox_manuscriptSize->count()-1));
    ui->widget_customFinishedSize->setVisible(ui->comboBox_finishedSize->currentIndex() == (ui->comboBox_finishedSize->count()-1));
    ui->spinBox_customResolution->setVisible(ui->comboBox_resolution->currentIndex() == (ui->comboBox_resolution->count()-1));
    ui->spinBox_customBleedWidth->setVisible(ui->comboBox_bleedWidth->currentIndex() == (ui->comboBox_bleedWidth->count()-1));

    ui->comboBox_manuscriptSize->setItemText(0, tr("Automatic")+QString(" (%1 mm x %2 mm)").arg(manuscriptWidth()).arg(manuscriptHeight()));

    if (ui->comboBox_manuscriptSize->currentIndex() == 0) {
        // Automatic selected
        ui->groupBox_cropMarkings->setEnabled(true);
    } else {
        ui->groupBox_cropMarkings->setEnabled((manuscriptWidth()>finishedWidth()) && (manuscriptHeight()>finishedHeight()));
    }
}
