#include "shapelayer.h"

#include <QUuid>

#include <quazip.h>
#include <quazipfile.h>
#include <quazipnewinfo.h>
#include <comicutils.h>

int ShapeLayer::global_counter;

struct ShapeLayer::Private {

};

ShapeLayer::ShapeLayer(const QString &name, float lineStrength, QObject *parent) :
    QObject(parent)
  , m_name(name)
  , m_counter(0)
  , m_uuid(QUuid::createUuid().toString())
{
    global_counter++;
    m_content = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
                "<office:document-content xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\" xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\" xmlns:config=\"urn:oasis:names:tc:opendocument:xmlns:config:1.0\" xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\" xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\" xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\" xmlns:presentation=\"urn:oasis:names:tc:opendocument:xmlns:presentation:1.0\" xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\" xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\" xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\" xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\" xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\" xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\" xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\" xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\" xmlns:anim=\"urn:oasis:names:tc:opendocument:xmlns:animation:1.0\" xmlns:smil=\"urn:oasis:names:tc:opendocument:xmlns:smil-compatible:1.0\" xmlns:calligra=\"http://www.calligra.org/2005/\" xmlns:officeooo=\"http://openoffice.org/2009/office\" xmlns:delta=\"http://www.deltaxml.com/ns/track-changes/delta-namespace\" xmlns:split=\"http://www.deltaxml.com/ns/track-changes/split-namespace\" xmlns:ac=\"http://www.deltaxml.com/ns/track-changes/attribute-change-namespace\" office:version=\"1.2\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n"
                " <office:automatic-styles>\n"
                "  <style:style style:name=\"gr0\" style:family=\"graphic\">\n"
                "   <style:graphic-properties draw:fill=\"none\" draw:shadow=\"hidden\" draw:shadow-color=\"#000000\" draw:shadow-offset-x=\"0pt\" draw:shadow-offset-y=\"0pt\" draw:shadow-opacity=\"0%\" draw:stroke=\"solid\" draw:stroke-linejoin=\"miter\" draw:textarea-horizontal-align=\"left\" draw:textarea-vertical-align=\"top\" fo:margin=\"0.000000000000000pt\" style:wrap=\"biggest\" style:wrap-contour=\"true\" style:wrap-contour-mode=\"full\" style:wrap-dynamic-threshold=\"0.000000000000000pt\" svg:fill-rule=\"evenodd\" svg:stroke-color=\"#000000\" svg:stroke-linecap=\"square\" svg:stroke-opacity=\"1\" svg:stroke-width=\"0.480000000000000pt\"/>\n"
                "  </style:style>\n"
                "  <style:style style:name=\"gr1\" style:family=\"graphic\">\n"
                "   <style:graphic-properties draw:fill=\"solid\" draw:fill-color=\"#000000\" draw:stroke=\"solid\" draw:stroke-linejoin=\"miter\" draw:textarea-horizontal-align=\"left\" draw:textarea-vertical-align=\"top\" fo:margin=\"0.000000000000000pt\" style:wrap=\"biggest\" style:wrap-contour=\"true\" style:wrap-contour-mode=\"full\" style:wrap-dynamic-threshold=\"0.000000000000000pt\" svg:fill-rule=\"evenodd\" svg:stroke-color=\"#000000\" svg:stroke-linecap=\"square\" svg:stroke-opacity=\"0\" svg:stroke-width=\"0.000000000000000pt\"/>\n"
                "  </style:style>\n"
                "  <style:style style:name=\"gr2\" style:family=\"graphic\">\n"
                "   <style:graphic-properties draw:fill=\"solid\" draw:fill-color=\"#ffffff\" draw:stroke=\"solid\" draw:stroke-linejoin=\"miter\" draw:textarea-horizontal-align=\"left\" draw:textarea-vertical-align=\"top\" fo:margin=\"0.000000000000000pt\" style:wrap=\"biggest\" style:wrap-contour=\"true\" style:wrap-contour-mode=\"full\" style:wrap-dynamic-threshold=\"0.000000000000000pt\" svg:fill-rule=\"evenodd\" svg:stroke-color=\"#000000\" svg:stroke-linecap=\"square\" svg:stroke-opacity=\"0\" svg:stroke-width=\"0.000000000000000pt\"/>\n"
                "  </style:style>\n"
                "  <style:style style:name=\"gr3\" style:family=\"graphic\">\n"
                "   <style:graphic-properties draw:fill=\"solid\" draw:fill-color=\"#ffffff\" draw:shadow=\"hidden\" draw:shadow-color=\"#000000\" draw:shadow-offset-x=\"0pt\" draw:shadow-offset-y=\"0pt\" draw:shadow-opacity=\"0%\" draw:stroke=\"solid\" draw:stroke-linejoin=\"miter\" draw:textarea-horizontal-align=\"left\" draw:textarea-vertical-align=\"top\" fo:margin=\"0.000000000000000pt\" style:wrap=\"biggest\" style:wrap-contour=\"true\" style:wrap-contour-mode=\"full\" style:wrap-dynamic-threshold=\"0.000000000000000pt\" svg:fill-rule=\"evenodd\" svg:stroke-color=\"#000000\" svg:stroke-linecap=\"square\" svg:stroke-opacity=\"1\" svg:stroke-width=\"" + QString::number(ComicUtils::mmToPt(lineStrength)) +  "pt\"/>\n"
                "  </style:style>\n"
                " </office:automatic-styles>\n"
                " <office:body>\n"
                "  <office:drawing>\n"
                "   <draw:page draw:name=\"\" xml:id=\"page-1\" draw:id=\"page-1\" draw:master-page-name=\"Default\">\n";
}

ShapeLayer::~ShapeLayer()
{

}

QString ShapeLayer::content_xml() const
{
    return m_content + "   </draw:page>\n  </office:drawing>\n </office:body>\n</office:document-content>";
}

QString ShapeLayer::maindoc_xml() const
{
    return QString("<layer filename=\"layer%1\" collapsed=\"0\" x=\"0\" uuid=\"%2\" selected=\"true\" compositeop=\"normal\" channelflags=\"\" colorlabel=\"0\" name=\"%3\" opacity=\"255\" nodetype=\"shapelayer\" locked=\"1\" visible=\"1\" y=\"0\"/>\n").arg(global_counter).arg(uuid()).arg(m_name);
}

QString ShapeLayer::styles_xml() const
{
    return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
           "<office:document-styles xmlns:office=\"urn:oasis:names:tc:opendocument:xmlns:office:1.0\" xmlns:meta=\"urn:oasis:names:tc:opendocument:xmlns:meta:1.0\" xmlns:config=\"urn:oasis:names:tc:opendocument:xmlns:config:1.0\" xmlns:text=\"urn:oasis:names:tc:opendocument:xmlns:text:1.0\" xmlns:table=\"urn:oasis:names:tc:opendocument:xmlns:table:1.0\" xmlns:draw=\"urn:oasis:names:tc:opendocument:xmlns:drawing:1.0\" xmlns:presentation=\"urn:oasis:names:tc:opendocument:xmlns:presentation:1.0\" xmlns:dr3d=\"urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0\" xmlns:chart=\"urn:oasis:names:tc:opendocument:xmlns:chart:1.0\" xmlns:form=\"urn:oasis:names:tc:opendocument:xmlns:form:1.0\" xmlns:script=\"urn:oasis:names:tc:opendocument:xmlns:script:1.0\" xmlns:style=\"urn:oasis:names:tc:opendocument:xmlns:style:1.0\" xmlns:number=\"urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0\" xmlns:math=\"http://www.w3.org/1998/Math/MathML\" xmlns:svg=\"urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0\" xmlns:fo=\"urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0\" xmlns:anim=\"urn:oasis:names:tc:opendocument:xmlns:animation:1.0\" xmlns:smil=\"urn:oasis:names:tc:opendocument:xmlns:smil-compatible:1.0\" xmlns:calligra=\"http://www.calligra.org/2005/\" xmlns:officeooo=\"http://openoffice.org/2009/office\" xmlns:delta=\"http://www.deltaxml.com/ns/track-changes/delta-namespace\" xmlns:split=\"http://www.deltaxml.com/ns/track-changes/split-namespace\" xmlns:ac=\"http://www.deltaxml.com/ns/track-changes/attribute-change-namespace\" office:version=\"1.2\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n"
           " <office:styles/>\n"
           " <office:automatic-styles/>\n"
           "</office:document-styles>";
}

QString ShapeLayer::manifest_xml() const
{
    return "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
           "<manifest:manifest xmlns:manifest=\"urn:oasis:names:tc:opendocument:xmlns:manifest:1.0\" manifest:version=\"1.2\">\n"
           " <manifest:file-entry manifest:media-type=\"application/vnd.oasis.opendocument.graphics\" manifest:full-path=\"/\"/>\n"
           " <manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"content.xml\"/>\n"
           " <manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"styles.xml\"/>\n"
           " <manifest:file-entry manifest:media-type=\"text/xml\" manifest:full-path=\"settings.xml\"/>\n"
           "</manifest:manifest>";
}

void ShapeLayer::drawRect(QRectF rect, int style)
{
    m_content += QString("    <draw:rect draw:style-name=\"gr%2\" xml:id=\"shape-%1\" draw:id=\"shape-%1\" draw:layer=\"\" svg:width=\"%5pt\" svg:height=\"%6pt\" svg:x=\"%3pt\" svg:y=\"%4pt\">\n"
                         "     <draw:glue-point draw:id=\"4\" svg:x=\"0.50000000000pt\" svg:y=\"0.50000000000pt\" draw:align=\"center\"/>\n"
                         "    </draw:rect>\n").arg(m_counter).arg(style).arg(rect.x()).arg(rect.y()).arg(rect.width()).arg(rect.height());
    m_counter++;
}

void ShapeLayer::drawScaledBox(const QRectF &page, const QRectF &rect)
{
    m_content += QString("    <draw:rect draw:style-name=\"gr%2\" xml:id=\"shape-%1\" draw:id=\"shape-%1\" draw:layer=\"\" svg:width=\"%5pt\" svg:height=\"%6pt\" svg:x=\"%3pt\" svg:y=\"%4pt\">\n"
                         "     <draw:glue-point draw:id=\"4\" svg:x=\"0.50000000000pt\" svg:y=\"0.50000000000pt\" draw:align=\"center\"/>\n"
                         "    </draw:rect>\n").arg(m_counter).arg(3).arg(rect.x()*page.width()+page.x()+1).arg(rect.y()*page.height()+page.y()+1).arg(rect.width()*page.width()-2).arg(rect.height()*page.height()-2);
    m_counter++;
}

void ShapeLayer::writeFiles(QuaZip *quazip, const QString &title)
{
    QuaZipFile outFile(quazip);

    outFile.open(QIODevice::WriteOnly, QuaZipNewInfo(QString("%1/layers/layer%2.shapelayer/styles.xml").arg(title).arg(global_counter)));
    outFile.write(styles_xml().toUtf8());
    outFile.close();

    outFile.open(QIODevice::WriteOnly, QuaZipNewInfo(QString("%1/layers/layer%2.shapelayer/content.xml").arg(title).arg(global_counter)));
    outFile.write(content_xml().toUtf8());
    outFile.close();

    outFile.open(QIODevice::WriteOnly, QuaZipNewInfo(QString("%1/layers/layer%2.shapelayer/META-INF/manifest.xml").arg(title).arg(global_counter)));
    outFile.write(manifest_xml().toUtf8());
    outFile.close();
}

QString ShapeLayer::uuid() const
{
    return m_uuid;
}
