#ifndef COMICLAYOUT_H
#define COMICLAYOUT_H

#include <QObject>
#include <QSharedPointer>

class QSettings;
class QPixmap;
class QJsonArray;

class ComicLayout : public QObject
{
    Q_OBJECT

public:
    enum Orientation { Portrait, Landscape };
    Q_ENUM(Orientation)

    explicit ComicLayout(QObject *parent = 0);
    explicit ComicLayout(QString layoutFilename, Orientation orientation = Portrait, QObject *parent = 0);
    ComicLayout(const ComicLayout &other);
    ComicLayout &operator=(const ComicLayout &other);
    ~ComicLayout();

    void save(QSharedPointer<QSettings> conf);
    static QSharedPointer<ComicLayout> load(QSharedPointer<QSettings> conf);
    bool load(QByteArray json);
    bool load();

    QJsonArray layout() const;

    QPixmap preview() const;
    static QPixmap preview(QJsonArray layout, int width, int height);
    static QPixmap preview(QString filename, int width, int height);

    static void addLayoutDirectory(const QString &directory);
    static const QStringList &layouts();
    static const QMap<QString,QStringList> tags();


// Properties
public:
    Q_PROPERTY(QString filename READ filename WRITE setFilename NOTIFY filenameChanged)
    Q_PROPERTY(Orientation orientation READ orientation WRITE setOrientation NOTIFY orientationChanged)
    Q_PROPERTY(float margins READ margins WRITE setMargins NOTIFY marginsChanged)
    Q_PROPERTY(float lineStrength READ lineStrength WRITE setLineStrength NOTIFY lineStrengthChanged)

    QString filename() const;
    Orientation orientation() const;
    float margins() const;
    float lineStrength() const;

signals:
    void filenameChanged(QString filename);
    void orientationChanged(Orientation orientation);
    void marginsChanged(float margins);
    void marginsChanged(double margins);
    void lineStrengthChanged(float lineStrength);
    void lineStrengthChanged(double lineStrength);

public slots:
    void setFilename(QString filename);
    void setOrientation(Orientation orientation);
    void setMargins(float margins);
    void setMargins(double margins);
    void setLineStrength(float lineStrength);
    void setLineStrength(double lineStrength);

// Private
private:
    struct Private;
    QSharedPointer<Private> m_private;
};

#endif // COMICLAYOUT_H
