#ifndef COMICPAGE_H
#define COMICPAGE_H

#include <QObject>
#include <QSharedPointer>
#include <QSize>

#include "comiclayout.h"

class QSettings;

class ComicPage : public QObject
{
    Q_OBJECT

public:
    explicit ComicPage(QObject *parent = 0);
    explicit ComicPage(QString layoutFilename, ComicLayout::Orientation orientation = ComicLayout::Portrait, QObject *parent = 0);
    ComicPage(const ComicPage &other);
    ComicPage &operator=(const ComicPage &other);
    ~ComicPage();

    void save(QSharedPointer<QSettings> conf);
    static QSharedPointer<ComicPage> load(QSharedPointer<QSettings> conf);

    int index() const;


// Properties
public:
    Q_PROPERTY(QSharedPointer<ComicLayout> layout READ layout WRITE setLayout NOTIFY layoutChanged)
    Q_PROPERTY(QSize bindingSize READ bindingSize WRITE setBindingSize NOTIFY bindingSizeChanged)
    Q_PROPERTY(QSize manuscriptSize READ manuscriptSize WRITE setManuscriptSize NOTIFY manuscriptSizeChanged)
    Q_PROPERTY(int bleedWidth READ bleedWidth WRITE setBleedWidth NOTIFY bleedWidthChanged)
    Q_PROPERTY(int resolution READ resolution WRITE setResolution NOTIFY resolutionChanged)

    QSharedPointer<ComicLayout> layout() const;
    QSize bindingSize() const;
    QSize manuscriptSize() const;
    int bleedWidth() const;
    int resolution() const;

signals:
    void layoutChanged(QSharedPointer<ComicLayout> layout);
    void bindingSizeChanged(QSize bindingSize);
    void manuscriptSizeChanged(QSize manuscriptSize);
    void bleedWidthChanged(int bleedWidth);
    void resolutionChanged(int resolution);

public slots:
    void setLayout(QSharedPointer<ComicLayout> layout);
    void setBindingSize(QSize bindingSize);
    void setManuscriptSize(QSize manuscriptSize);
    void setBleedWidth(int bleedWidth);
    void setResolution(int resolution);


// Private
private:
    struct Private;
    QSharedPointer<Private> m_private;
};

#endif // COMICPAGE_H
