#ifndef COMICPROJECT_H
#define COMICPROJECT_H

#include <QObject>
#include <QSharedPointer>

#include "comiclayout.h"

class QSettings;
class ComicPage;

class ComicProject : public QObject
{
    Q_OBJECT

public:
    explicit ComicProject(QObject *parent = 0);
    ComicProject(const ComicProject &other);
    ComicProject &operator=(const ComicProject &other);
    ~ComicProject();

    bool save(QString foldername = QString());
    static QSharedPointer<ComicProject> load(QString foldername);
    void save(QSharedPointer<QSettings> conf);
    static QSharedPointer<ComicProject> load(QSharedPointer<QSettings> conf);

    QSharedPointer<ComicPage> insertPage(QSharedPointer<ComicPage> page, int index = -1);
    QSharedPointer<ComicPage> insertPage(ComicPage *page, int index = -1);
    QSharedPointer<ComicPage> insertPage(QString layoutFilename, ComicLayout::Orientation orientation = ComicLayout::Portrait, int index = -1);
    void removePage(QSharedPointer<ComicPage> page);
    void removePage(int index);

    QList< QSharedPointer<ComicPage> > pages() const;

signals:
    void changed();


// Properties
public:
    Q_PROPERTY(QString foldername READ foldername WRITE setFoldername NOTIFY foldernameChanged)
    Q_PROPERTY(QString author READ author WRITE setAuthor NOTIFY authorChanged)
    Q_PROPERTY(QString title READ title WRITE setTitle NOTIFY titleChanged)

    QString foldername() const;
    QString author() const;
    QString title() const;

signals:
    void foldernameChanged(QString foldername);
    void authorChanged(QString author);
    void titleChanged(QString title);

public slots:
    void setFoldername(QString foldername);
    void setAuthor(QString author);
    void setTitle(QString title);


// Private
private:
    struct Private;
    QSharedPointer<Private> m_private;
};

#endif // COMICPROJECT_H
